
#include <avr/interrupt.h>
#include <util/delay.h>


#include "./apps.h"
#include "./clocktimer.h"
#include "../types/mytypes.h"
#include "../types/ranged.h"
#include "../menu/menu.h"
#include "../menu/displayroutines.h"
#include "./realtimeclock.h"

#define FIRSTENTRY 0
#define EDIT 1
#define CHANCEL 2
#define SAVE 3
#define DELETE 4
#define ERROR 5

uint8_t myState = FIRSTENTRY;

static uint8_t DisplayErrorSeconds = 3;

static t_Switchpoint mySwitchpoint;

//char* SwitchpointToASCII(const t_Switchpoint *aSwitchpoint){
//    static char result[15];
//    uint8_t i = 7;
//    t_ranged_uint8_t index = {
//         .value = FIRSTDAYOFWEEK,
//         .top = 6,
//         .bottom = 0
//    };
//    result[0] = '\0';
//    result[14] = '\0';
//    do{
//        if(aSwitchpoint->days & (1 << index.value)){
//            strncat(result, daynames[index.value],1);
//        }
//        else{
//            strcat(result, "X");
//        }
//        rangedInc_uint8_t(&index);
//    }while(--i);
//    strcat(result, _2DigitValueWithLeadingNull((uint8_t *)&aSwitchpoint->hour));
//    strcat(result, ":");
//    strcat(result, _2DigitValueWithLeadingNull((uint8_t *)&aSwitchpoint->minute));
//    result[12] = ' ';
//    if( ((1<<FLAG_CHANNEL_HEIZUNG) & aSwitchpoint->flags) && ((1<<FLAG_CHANNEL_WARMWASSER) & aSwitchpoint->flags) ){
//         result[12] = 'B';
//    }
//    else{
//        if(aSwitchpoint->flags & (1<<FLAG_CHANNEL_HEIZUNG)){ result[12] = 'H';}
//        if(aSwitchpoint->flags & (1<<FLAG_CHANNEL_WARMWASSER)){ result[12] = 'W';}
//    }
//    if(aSwitchpoint->flags & (1<<FLAG_ACTION)){
//         result[13] = 'o';
//    }
//    else{
//        result[13] = 'x';
//    }
//    if(! (aSwitchpoint->flags & (1<<FLAG_ISVALID))){
//        result[13] = '-';
//    }
//    return result;
//}

char* getItemText(uint8_t ItemType, uint8_t index){
    static char buffer[13];
    buffer[0] = '\0';
    readSwitchpoint(index, &mySwitchpoint);
    if(!(mySwitchpoint.flags & (1<<FLAG_ISVALID))){ //Switchpoint invalid!!!!
        if(ItemType == 1){ //ItemType fr Menu == 1 u
            utoa(index + 1, buffer, 10);
            strcat(buffer, " ist leer");
            return buffer;
        }
        else{
            mySwitchpoint.days = 0;
            mySwitchpoint.hour = 0;
            mySwitchpoint.minute = 0;
            mySwitchpoint.flags = 0;
        }
    }
    return SwitchpointToASCII(&mySwitchpoint);
}



t_ranged_uint8_t SetChannelCounter = {
    .value = 0,
    .bottom = 1,
    .top = 3
};

static t_ranged_uint8_t SetPosition = {
        .value = 0,
        .top = 10,
        .bottom = 0,
};

void cleanup(void){
    DisplayErrorSeconds = 3;
    SetPosition.value = 0;
    myState = FIRSTENTRY;
    ActiveApplication = &MenuApplication;
    ExitItem();
}

void UpDownEditHandler(){
    switch(SetPosition.value){
        case 0:{
            mySwitchpoint.days ^=  (1<<MONTAG);
        break;
        }
        case 1:{
            mySwitchpoint.days ^=  (1<<DIENSTAG);
        break;
        }
        case 2:{
            mySwitchpoint.days ^=  (1<<MITTWOCH);
        break;
        }
        case 3:{
            mySwitchpoint.days ^=  (1<<DONNERSTAG);
        break;
        }
        case 4:{
            mySwitchpoint.days ^=  (1<<FREITAG);
        break;
        }
        case 5:{
            mySwitchpoint.days ^=  (1<<SAMSTAG);
        break;
        }
        case 6:{
            mySwitchpoint.days ^=  (1<<SONNTAG);
        }
        break;
        case 10:{
            mySwitchpoint.flags ^= (1<<FLAG_ACTION);
        }
    }
}

void EditSwitchpointApp(uint8_t Event){
    t_datetime datetime;
    datetime.hour = mySwitchpoint.hour;
    datetime.minute = mySwitchpoint.minute;
    SetChannelCounter.value = mySwitchpoint.flags & ((1<<FLAG_CHANNEL_HEIZUNG) | (1<<FLAG_CHANNEL_WARMWASSER));
    switch( Event){
     case OK:{
        switch(myState){
            case EDIT:
                myState = CHANCEL;
                EditSwitchpointApp(NO_KEY);
            break;
            case CHANCEL:
                cleanup();
                return;
            break;
            case SAVE:
                //save to eeprom
                if(mySwitchpoint.days && (mySwitchpoint.flags & ((1<<FLAG_CHANNEL_HEIZUNG) | (1<<FLAG_CHANNEL_WARMWASSER)))){
                    //writeSwitchpoint(menu_getCurrentItemId(), mySwitchpoint);
                    cleanup();
                }
                else{
                    Beep();
                    PrintStringPos(0,2, "You must set");
                    PrintStringPos(0,3, "day + channel");
                    myState = ERROR;
                }
                return;
            break;
            case DELETE:
                //delete in eeprom
                //deleteSwitchpoint(menu_getCurrentItemId());
                cleanup();
                return;
            break;
        }
     }
     break;
     case UP:{
        switch(myState){
            case EDIT:{
                switch(SetPosition.value){
                    case 0:
                    case 1:
                    case 2:
                    case 3:
                    case 4:
                    case 5:
                    case 6:
                    case 10:
                        UpDownEditHandler();
                    break;
                    case 7:{
                        RealTime_IncrementHour(&datetime);
                    }
                    break;
                    case 8:{
                        RealTime_IncrementMinute(&datetime);
                    }
                    break;
                    case 9:{
                        rangedInc_uint8_t(&SetChannelCounter);
                        mySwitchpoint.flags &=~((1<<FLAG_CHANNEL_HEIZUNG) | (1<<FLAG_CHANNEL_WARMWASSER));
                        mySwitchpoint.flags |=SetChannelCounter.value;
                    }
                    break;
                }
            }
            break;
            case CHANCEL:
                myState = DELETE;
                EditSwitchpointApp(NO_KEY);
            break;
            case SAVE:
                myState = CHANCEL;
                EditSwitchpointApp(NO_KEY);
            break;
            case DELETE:
                myState = SAVE;
                EditSwitchpointApp(NO_KEY);
            break;
        }
     }
     break;
     case DOWN:{
        switch(myState){
            case EDIT:{
                switch(SetPosition.value){
                    case 0:
                    case 1:
                    case 2:
                    case 3:
                    case 4:
                    case 5:
                    case 6:
                    case 10:
                        UpDownEditHandler();
                    break;
                    case 7:{
                        RealTime_DecrementHour(&datetime);
                    }
                    break;
                    case 8:{
                        RealTime_DecrementMinute(&datetime);
                    }
                    break;
                    case 9:{
                        rangedDec_uint8_t(&SetChannelCounter);
                        mySwitchpoint.flags &=~((1<<FLAG_CHANNEL_HEIZUNG) | (1<<FLAG_CHANNEL_WARMWASSER));
                        mySwitchpoint.flags |=SetChannelCounter.value;
                    }
                    break;
                }
            }
            break;
            case CHANCEL:
                myState = SAVE;
                EditSwitchpointApp(NO_KEY);

            break;
            case SAVE:
                myState = DELETE;
                EditSwitchpointApp(NO_KEY);

            break;
            case DELETE:
                myState = CHANCEL;
                EditSwitchpointApp(NO_KEY);
            break;
        }
     }
     break;
     case LEFT:{
        switch(myState){
            case EDIT:
                rangedDec_uint8_t(&SetPosition);
            break;
        }
     }
    break;
    case RIGHT:{
        switch(myState){
            case EDIT:
                rangedInc_uint8_t(&SetPosition);
            break;
        }
    }
    break;
    case NO_KEY:{
        switch(myState){
            case CHANCEL:
                Clear();
                Highlight(2);
                PrintString( "CHANCEL?");
            break;
            case SAVE:
                Clear();
                Highlight(2);
                PrintString( "SAVE?");
            break;
            case DELETE:
                Clear();
                Highlight(2);
                PrintString("DELETE?");
            break;
        }
    }
    break;
    case TICK:
        switch(myState){
            case ERROR:
                if(--DisplayErrorSeconds) break;
                DisplayErrorSeconds = 3;
                myState = EDIT;
            break;
        }
    default:
    break;
    }
    switch(myState){
        case FIRSTENTRY:
            //loads mySwitchpoint persistent with current item
            PrintStringPos(0,2, getItemText(1,menu_getCurrentItemId()));
            mySwitchpoint.flags |= (1<<FLAG_ISVALID);
            myState = EDIT;
        break;
        case EDIT:
            mySwitchpoint.hour = datetime.hour;
            mySwitchpoint.minute = datetime.minute;
            PrintStringPos(0,2, SwitchpointToASCII(&mySwitchpoint));
            char *HighlightChar = "^";
            uint8_t HighlightPosition = SetPosition.value;
            if ((SetPosition.value == 7) | (SetPosition.value == 8)){
                HighlightChar = "^^";
            }
            if(SetPosition.value == 8){
                HighlightPosition = SetPosition.value + 2;
            }
            else if (SetPosition.value > 8){
                HighlightPosition = SetPosition.value + 3;
            }
            PrintStringPos(0, 3, "              ");
            PrintStringPos(HighlightPosition, 3, HighlightChar);
        break;
    }
}
