
#include <stdlib.h>

#include "./realtimeclock.h"
#include "../types/ranged.h"

const char daynames[7][3] = {"So\0", "Mo\0", "Di\0", "Mi\0", "Do\0", "Fr\0", "Sa\0"};

//t_datetime mydatetime = {
//
//    .second = 0,			// count seconds
//    .minute = 0,
//    .hour = 13,
//    .day = 2,
//    .month = 8,
//    .year = 2009,  //bis 2047
//    .dow = 0,    //Day of Week 0 for Sunday
//    .dayname = "So\0",
//};

t_datetime mydatetime = {

    .second = 59,			// count seconds
    .minute = 28,
    .hour = 5,
    .day = 2,
    .month = 8,
    .year = 2009,  //bis 2047
    .dow = 0,    //Day of Week 0 for Sunday
    .dayname = "So\0",
};

uint8_t correctDSTinOctober = 1;
uint8_t bool_is_leapyear = 0;
const uint8_t days_in_months[2][12] = {
    {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
    {31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}
};



t_ranged_uint8_t rangedMinuteAndSecond = {
        .value = 0,
        .top = 59,
        .bottom = 0
};

t_ranged_uint8_t rangedHour = {
        .value = 0,
        .top = 23,
        .bottom = 0
};

uint8_t is_leapyear(uint16_t ayear){
    if (((ayear % 4 == 0) && (ayear % 100 != 0)) || (ayear % 400 == 0)){
        return 1;
    }
    return 0;
}

void RealTime_OneSecondOver(void){
    mydatetime.second++;				// exact one second over
    if(mydatetime.second == 60){
        mydatetime.second = 0;
        mydatetime.minute++;
        OneMinuteTick();
        if(mydatetime.minute == 60){
            mydatetime.minute = 0;
            mydatetime.hour++;
            if(mydatetime.hour >= 24){
                mydatetime.hour = 0;

                if (++mydatetime.dow >= 7){
                    mydatetime.dow = 0;
                }
                strcpy(mydatetime.dayname, daynames[mydatetime.dow]);
                if (++mydatetime.day > days_in_months[bool_is_leapyear][mydatetime.month - 1]){
                    mydatetime.month++;
                    mydatetime.day = 1;
                }
                if (mydatetime.month >= 13){
                    mydatetime.month = 1;
                    mydatetime.year++;
                    bool_is_leapyear = is_leapyear(mydatetime.year);
                }
            }
        }
        if (mydatetime.month <3 || mydatetime.month >10){
            return;
        }
        else{
            if (mydatetime.day >=25 && mydatetime.dow == 0){
                if(mydatetime.month == 3 && mydatetime.hour == 2){
                    correctDSTinOctober = 1;
                    mydatetime.hour++;
                    return;
                }
                if(mydatetime.month == 10 && mydatetime.hour == 3 && correctDSTinOctober){
                    correctDSTinOctober = 0;
                    mydatetime.hour--;
                    return;
                }
            }
        }
    }
}

/*elegant code by Tomohiko Sakamoto*/
/*found in www: http://c-faq.com/misc/zeller.html*/
uint8_t dayofweek(uint16_t ayear, uint8_t amonth, uint8_t aday)	/* 0 = Sunday */
{
	static uint8_t t[] = {0, 3, 2, 5, 0, 3, 5, 1, 4, 6, 2, 4};
	ayear -= amonth < 3;
	return (ayear + ayear/4 - ayear/100 + ayear/400 + t[amonth-1] + aday) % 7;
}

t_datetime RealTime_GetDateTime(void){
    return mydatetime;
}

void RealTime_SetTime(const t_datetime *time){
    mydatetime.hour = time->hour;
    mydatetime.minute = time->minute;
    mydatetime.second = time->second;
}

void RealTime_IncrementHour(t_datetime *time){
    rangedHour.value = time->hour;
    rangedInc_uint8_t(&rangedHour);
    time->hour = rangedHour.value;
}

void RealTime_DecrementHour(t_datetime *time){
    rangedHour.value = time->hour;
    rangedDec_uint8_t(&rangedHour);
    time->hour = rangedHour.value;
}

void RealTime_IncrementMinute(t_datetime *time){
    rangedMinuteAndSecond.value = time->minute;
    rangedInc_uint8_t(&rangedMinuteAndSecond);
    time->minute = rangedMinuteAndSecond.value;
}

void RealTime_DecrementMinute(t_datetime *time){
    rangedMinuteAndSecond.value = time->minute;
    rangedDec_uint8_t(&rangedMinuteAndSecond);
    time->minute = rangedMinuteAndSecond.value;
}

void RealTime_IncrementSecond(t_datetime *time){
    rangedMinuteAndSecond.value = time->second;
    rangedInc_uint8_t(&rangedMinuteAndSecond);
    time->second = rangedMinuteAndSecond.value;
}

void RealTime_DecrementSecond(t_datetime *time){
    rangedMinuteAndSecond.value = time->second;
    rangedDec_uint8_t(&rangedMinuteAndSecond);
    time->second = rangedMinuteAndSecond.value;
}

void RealTime_SetDate(const t_datetime *date){
    mydatetime.day = date->day;
    mydatetime.month = date->month;
    mydatetime.year = date->year;
    bool_is_leapyear = is_leapyear(mydatetime.year);
    mydatetime.dow = dayofweek(mydatetime.year, mydatetime.month, mydatetime.day);
    strcpy(mydatetime.dayname, daynames[mydatetime.dow]);
}

void RealTime_SetDateTime(const t_datetime *datetime){
    correctDSTinOctober = 1;
    RealTime_SetTime(datetime);
    RealTime_SetDate(datetime);
}

void correctforlastDay(t_datetime *date){
    uint8_t lastDay = days_in_months[is_leapyear(date->year)][date->month - 1];
    if (date->day > lastDay){
        date->day = lastDay;
    }
}

void RealTime_IncrementDay(t_datetime *date){
    if (++date->day > days_in_months[is_leapyear(date->year)][date->month - 1]){
        date->day = 1;
    }
}

void RealTime_IncrementMonth(t_datetime *date){
    if (++date->month > 12){
        date->month = 1;
    }
    correctforlastDay(date);
}

void RealTime_IncrementYear(t_datetime *date){
    ++date->year;
    correctforlastDay(date);
}

void RealTime_DecrementDay(t_datetime *date){
    if (--date->day == 0){
        date->day = days_in_months[is_leapyear(date->year)][date->month - 1];
    }
}

void RealTime_DecrementMonth(t_datetime *date){
    if (--date->month == 0){
        date->month = 12;
    }
    correctforlastDay(date);
}

void RealTime_DecrementYear(t_datetime *date){
    --date->year;
    correctforlastDay(date);
}


